--
-- Pixel art rendering example.
-- Shaders could be used but aren't in this example.
--

local CAMERA_WIDTH  = 256
local CAMERA_HEIGHT = 223

local canvas, canvasFilter, canvasSubpixels
local theGameWorld
local integerScaling = false

local function createCanvas(filter, subpixels)
	canvasFilter    = filter
	canvasSubpixels = subpixels

	canvas = love.graphics.newCanvas(CAMERA_WIDTH*canvasSubpixels, CAMERA_HEIGHT*canvasSubpixels)
	canvas:setFilter(canvasFilter, canvasFilter)
end

function love.load()
	love.graphics.setDefaultFilter("nearest", "nearest")
	theGameWorld = love.graphics.newImage("superMetroid.png")
	createCanvas("nearest", 1)
end

function love.keypressed(key)
	if key == "1" then
		-- Sharpest pixels, but pixels will likely be different sizes.
		createCanvas("nearest", 1)

	elseif key == "2" then
		-- Linear filter with no subpixels will usually result in ugly blur.
		createCanvas("linear", 1)

	elseif key == "3" then
		-- Use subpixels with linear filter for a little anti-aliasing so
		-- pixel sizes look more uniform, but everything is still pretty sharp.
		createCanvas("linear", 2)

	elseif key == "4" then
		-- Larger difference between camera and window size require more
		-- subpixels to prevent blurriness. (The subpixel amount could be
		-- calculated dynamically from the window height.)
		createCanvas("linear", 4)

	elseif key == "i" then
		integerScaling = not integerScaling
		-- Note: Subpixels are not needed if integer scaling is enabled.

	elseif key == "escape" then
		love.event.quit()
	end
end

function love.draw()
	--
	-- Render game world.
	--
	love.graphics.push("all")
	love.graphics.setCanvas(canvas)
	love.graphics.clear()
	love.graphics.scale(canvasSubpixels)

	-- Render sprites. (Here we're just using a screenshot as a substitute.)
	love.graphics.draw(theGameWorld)

	love.graphics.pop()

	--
	-- Render canvas.
	--
	local windowWidth, windowHeight = love.graphics.getDimensions()
	local canvasWidth, canvasHeight = canvas:getDimensions()

	-- Fill as much of the window as possible with the canvas while preserving the aspect ratio.
	local canvasScale = math.min(windowWidth/canvasWidth, windowHeight/canvasHeight)
	-- local canvasScale = windowHeight/canvasHeight -- This would fill the height and possibly cut off the sides.

	if integerScaling then
		canvasScale = math.floor(canvasScale * canvasSubpixels) / canvasSubpixels
		canvasScale = math.max(canvasScale, 1/canvasSubpixels) -- Avoid canvasScale=0 if the window is tiny!
	end

	local canvasWidthScaled  = canvasWidth  * canvasScale
	local canvasHeightScaled = canvasHeight * canvasScale

	local canvasX = math.floor((windowWidth  - canvasWidthScaled ) / 2) -- Center canvas.
	local canvasY = math.floor((windowHeight - canvasHeightScaled) / 2)

	love.graphics.clear(.2, .2, .2)
	love.graphics.draw(canvas, canvasX, canvasY, 0, canvasScale)

	local text
		=  "filter = " .. canvasFilter
		.. "\nsubpixels = " .. canvasSubpixels .. "x" .. canvasSubpixels
		.. "\ninteger scaling = " .. (integerScaling and "on" or "off")
		.. "\ncanvas scale = " .. string.format("%.2f", canvasScale)
		.. "\n"
		.. "\n1: no subpixels, nearest filter"
		.. "\n2: no subpixels, linear filter"
		.. "\n3: few subpixels"
		.. "\n4: more subpixels"
		.. "\ni: toggle integer scaling"
		.. "\n"
		.. "\nTry settings at different window sizes!"
	love.graphics.print(text, 5, 5)
end
